/**
 * Run api server
 * @function apiServer
 * @pram {Object} $api
 * @param {Object} [options={}] - Optional settings
 * @returns {Promise}
 */
'use strict'

const co = require('co')
const http = require('http')
const rfuncEndpoints = require('./rfunc_endpoints')

/** @lends apiServer */
function apiServer ($api, options = {}) {
  return co(function * () {
    if (!$api) {
      throw new Error('$api is required')
    }
    $api = Object.assign({ $pathname: '/api' }, $api)
    let server = http.createServer()

    rfuncEndpoints($api).attachTo(server)

    let { listen, close } = server
    return Object.assign(server, {
      $pathname: $api.$pathname,
      // Wrap listen to promise
      listen (port) {
        return new Promise((resolve, reject) =>
          listen.call(server, port, (err) => err ? reject(err) : resolve())
        )
      },
      // Wrap close to promise
      close () {
        return new Promise((resolve, reject) =>
          close.call(server, (err) => err ? reject(err) : resolve())
        )
      }
    })
  })
}

Object.assign(apiServer, {})

module.exports = apiServer
