/**
 * Define rfunc server endpoints
 * @function rfuncEndpoints
 * @returns {RFunc} - A rfunc instance
 */
'use strict'

const { spinalcase } = require('stringcase')
const rfunc = require('rfunc')
const SCOPE_PREFIX = /^@/
const SETTING_PREFIX = /^\$/

/** @lends rfuncEndpoints */
function rfuncEndpoints (modules) {
  let {
    base,
    scoped,
    settings
  } = Object.keys(modules).reduce((formed, key) => {
    let value = modules[ key ]
    if (SETTING_PREFIX.test(key)) {
      formed.settings[ key ] = value
    } else if (SCOPE_PREFIX.test(key)) {
      formed.scoped[ key ] = value
    } else {
      formed.base[ key ] = value
    }
    return formed
  }, { base: {}, scoped: {}, settings: {} })

  return rfunc([
    (Object.assign({}, settings, base)),
    ...Object.keys(scoped).map((name) => {
      let $pathname = `${settings.$pathname}/scoped/${spinalcase(name.replace(SCOPE_PREFIX, '')).trim()}`
      return Object.assign({}, settings, { $pathname }, scoped[ name ])
    })
  ])
}

module.exports = rfuncEndpoints
