/**
 * Transaction with api.
 * @function transaction
 * @param {function} handler
 * @returns {function} - Runnable transaction
 */
'use strict'

const co = require('co')
const aport = require('aport')
const apemanApi = require('./apeman_api')
const apemanApiClient = require('../client')
const logger = require('./logger')

/** @lends transaction */
function transaction (handler) {
  function run (options = {}) {
    return co(function * () {
      let startAt = new Date()
      logger.point('Transaction started...')
      let { port, configuration, host } = options
      if (!port) {
        port = yield aport()
      }
      let server = yield apemanApi({ port, configuration })
      let { $pathname } = server
      try {
        let client = apemanApiClient(`http://${host || 'localhost'}:${port}${$pathname}`)
        yield handler(client)
      } catch (err) {
        yield server.close()
        throw err
      }
      yield server.close()
      let took = new Date() - startAt
      logger.point(`...transaction done! (${took}ms)`)
      return {
        took, port, $pathname
      }
    })
  }

  return Object.assign(run, { run })
}

module.exports = transaction
