/**
 * Test case for apemanApi.
 * Runs with mocha.
 */
'use strict'

const apemanApi = require('../lib/apeman_api.js')
const asleep = require('asleep')
const assert = require('assert')
const arequest = require('arequest')
const aport = require('aport')
const uuid = require('uuid')
const co = require('co')

describe('apeman-api', function () {
  this.timeout(3000)
  let request = arequest.create({ jar: true })
  let port, baseUrl
  before(() => co(function * () {
    port = yield aport()
    baseUrl = `http://localhost:${port}`
  }))

  after(() => co(function * () {
  }))

  it('Command', () => co(function * () {
    let configuration = require.resolve('../misc/mocks/mock-Apemanfile')
    let server = yield apemanApi({
      port,
      configuration
    })
    assert(server)
    // Describe for all api
    {
      let { statusCode, body, headers } = yield request({
        url: `${baseUrl}/api`,
        method: 'OPTIONS'
      })
      assert.equal(statusCode, 200)
      let spec = body.data.attributes
      assert.ok(spec)
    }

    // Use default scope
    {
      let { statusCode, body } = yield request({
        url: `${baseUrl}/api/sign/signin`,
        method: 'POST',
        json: true,
        body: {
          data: {
            type: 'invocations',
            id: uuid.v4(),
            attributes: {
              params: [ 'foo', 'bar' ]
            }
          }
        }
      })
      assert.equal(statusCode, 200)
      let { returns } = body.data.attributes
      assert.deepEqual(returns, { success: true })
    }

    // Use default scope with camel
    {
      let { statusCode, body } = yield request({
        url: `${baseUrl}/api/hoge-hoge/say-hey`,
        method: 'POST',
        json: true,
        body: {
          data: {
            type: 'invocations',
            id: uuid.v4(),
            attributes: {}
          }
        }
      })
      assert.equal(statusCode, 200)
      let { returns } = body.data.attributes
      assert.deepEqual(returns, 'hey')
    }

    // Use custom scope
    {
      let { statusCode, body } = yield request({
        url: `${baseUrl}/api/scoped/admin/user/destroy`,
        method: 'POST',
        json: true,
        body: {
          data: {
            type: 'invocations',
            id: uuid.v4(),
            attributes: {
              params: [ 'foo', 'bar' ]
            }
          }
        }
      })
      assert.equal(statusCode, 200)
      let { returns } = body.data.attributes
      assert.deepEqual(returns, { success: false })
    }

    yield asleep(300)
    yield server.close()
  }))
})

/* global describe, before, after, it */
